import { isPlatformBrowser, CommonModule } from '@angular/common';
import { Component, ViewEncapsulation, Inject, PLATFORM_ID, ElementRef, Renderer2, Input, NgModule } from '@angular/core';

class NtBadgeComponent {
    constructor(platformId, _elementRef, _renderer) {
        this.platformId = platformId;
        this._elementRef = _elementRef;
        this._renderer = _renderer;
        this._type = 'static';
        this.color = '';
        this.class = '';
    }
    set type(value) {
        if (value === 'float' || value === 'notify') {
            this._resetPosition();
        }
        else {
            this._clearPosition();
        }
        this._type = value;
    }
    get type() { return this._type; }
    ngAfterViewInit() {
        this._resetPosition();
    }
    ngOnDestroy() {
        this._clearPosition();
    }
    _resetPosition() {
        if (isPlatformBrowser(this.platformId)) {
            const style = window.getComputedStyle(this._elementRef.nativeElement.parentElement);
            if (style.position !== 'absolute' || style.position !== 'absolute') {
                this._originalPosition = style.position;
                this._renderer.setStyle(this._elementRef.nativeElement.parentElement, 'position', 'relative');
            }
        }
    }
    _clearPosition() {
        if (isPlatformBrowser(this.platformId)) {
            if (this._originalPosition) {
                this._renderer.removeStyle(this._elementRef.nativeElement.parentElement, 'position');
            }
        }
    }
}
NtBadgeComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-badge, [nt-badge]',
                template: `<ng-content></ng-content>`,
                encapsulation: ViewEncapsulation.None,
                host: {
                    '[class]': '["badge", color, class].join(" ")',
                    '[class.badge-float]': 'type === "float"',
                    '[class.badge-notify]': 'type === "notify"'
                }
            },] }
];
NtBadgeComponent.ctorParameters = () => [
    { type: Object, decorators: [{ type: Inject, args: [PLATFORM_ID,] }] },
    { type: ElementRef },
    { type: Renderer2 }
];
NtBadgeComponent.propDecorators = {
    color: [{ type: Input }],
    class: [{ type: Input }],
    type: [{ type: Input }]
};

class NtBadgeModule {
}
NtBadgeModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                declarations: [NtBadgeComponent],
                exports: [NtBadgeComponent]
            },] }
];

export { NtBadgeComponent, NtBadgeModule };
//# sourceMappingURL=badge.js.map
