import { CommonModule } from '@angular/common';
import { EventEmitter, Component, ViewEncapsulation, Input, Output, HostBinding, HostListener, NgModule } from '@angular/core';
import { trigger, transition } from '@angular/animations';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { fadeOut } from '@ng-tangram/components/core';

class NtCalloutComponent {
    constructor() {
        this.title = '';
        this.color = 'primary';
        this.size = 'medium';
        this.class = '';
        this._closed = false;
        this.close = new EventEmitter();
        this.fadeOut = false;
    }
    get closed() { return this._closed; }
    set closable(value) { this._closable = coerceBooleanProperty(value); }
    get closable() { return this._closable; }
    onAnimationDone(event) {
        if (event.toState) {
            this._closed = true;
        }
    }
    _close() {
        if (!this.fadeOut) {
            this.fadeOut = true;
            this.close.emit();
        }
    }
}
NtCalloutComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-callout',
                template: `
    <h3 *ngIf="title">{{title}}</h3>
    <ng-content></ng-content>
    <button *ngIf="this.closable" (click)="_close()" class="close-button" type="button">
      <span aria-hidden="true">&times;</span>
    </button>
  `,
                encapsulation: ViewEncapsulation.None,
                animations: [
                    trigger('fadeOut', [
                        transition('* => true', fadeOut(.3))
                    ])
                ],
                host: {
                    '[class]': '["callout nt-callout", color || "primary", size, class].join(" ")',
                    '[class.closed]': 'closed'
                }
            },] }
];
NtCalloutComponent.ctorParameters = () => [];
NtCalloutComponent.propDecorators = {
    title: [{ type: Input }],
    color: [{ type: Input }],
    size: [{ type: Input }],
    class: [{ type: Input }],
    closable: [{ type: Input }],
    close: [{ type: Output }],
    fadeOut: [{ type: HostBinding, args: ['@fadeOut',] }],
    onAnimationDone: [{ type: HostListener, args: ['@fadeOut.done', ['$event'],] }]
};

class NtCalloutModule {
}
NtCalloutModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                exports: [NtCalloutComponent],
                declarations: [NtCalloutComponent],
            },] }
];

export { NtCalloutComponent, NtCalloutModule };
//# sourceMappingURL=callout.js.map
