import { getSupportedInputTypes, Platform } from '@angular/cdk/platform';
import { CommonModule } from '@angular/common';
import { Component, ViewEncapsulation, Input, Directive, Optional, Self, ElementRef, NgModule } from '@angular/core';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { NgControl } from '@angular/forms';
import { NtFormFieldControl } from '@ng-tangram/components/forms';

class NtInputAddonComponent {
}
NtInputAddonComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-input-addon, [nt-input-addon]',
                template: `<ng-content></ng-content>`,
                encapsulation: ViewEncapsulation.None,
                host: {
                    'class': 'nt-input-addon input-group-label'
                }
            },] }
];

class NtInputGroupComponent {
    constructor() {
        this._transparent = false;
    }
    get transparent() { return this._transparent; }
    set transparent(value) {
        this._transparent = coerceBooleanProperty(value);
    }
}
NtInputGroupComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-input-group, [nt-input-group]',
                template: `<ng-content></ng-content>`,
                encapsulation: ViewEncapsulation.None,
                host: {
                    'class': 'nt-input-group input-group',
                    '[class.nt-input-group-transparent]': 'transparent'
                }
            },] }
];
NtInputGroupComponent.propDecorators = {
    transparent: [{ type: Input }]
};

class NtInputDirective extends NtFormFieldControl {
    constructor(ngControl, _platform, _elementRef) {
        super();
        this.ngControl = ngControl;
        this._platform = _platform;
        this._elementRef = _elementRef;
        this._disabled = false;
        this._type = 'text';
        this._readonly = false;
        this._required = false;
        this._focused = false;
        this.placeholder = '';
    }
    get disabled() {
        if (this.ngControl && this.ngControl.disabled !== null) {
            return this.ngControl.disabled;
        }
        return this._disabled;
    }
    set disabled(value) {
        this._disabled = coerceBooleanProperty(value);
        if (this._focused) {
            this._focused = false;
        }
    }
    get required() { return this._required; }
    set required(value) { this._required = coerceBooleanProperty(value); }
    get type() { return this._type; }
    set type(value) {
        this._type = value || 'text';
        if (!this._isTextarea() && getSupportedInputTypes().has(this._type)) {
            this._elementRef.nativeElement.type = this._type;
        }
    }
    get value() { return this._elementRef.nativeElement.value; }
    set value(value) {
        if (value !== this.value) {
            this._elementRef.nativeElement.value = value;
        }
    }
    get readonly() { return this._readonly; }
    set readonly(value) { this._readonly = coerceBooleanProperty(value); }
    get empty() { return !this._elementRef.nativeElement.value && !this._isBadInput(); }
    focus() { this._elementRef.nativeElement.focus(); }
    _focusChanged(isFocused) {
        if (isFocused !== this._focused && !this.readonly) {
            this._focused = isFocused;
        }
    }
    _isBadInput() {
        const validity = this._elementRef.nativeElement.validity;
        return validity && validity.badInput;
    }
    _isTextarea() {
        const nativeElement = this._elementRef.nativeElement;
        const nodeName = this._platform.isBrowser ? nativeElement.nodeName : nativeElement.name;
        return nodeName ? nodeName.toLowerCase() === 'textarea' : false;
    }
}
NtInputDirective.decorators = [
    { type: Directive, args: [{
                selector: 'input[ntInput], textarea[ntInput]',
                host: {
                    '(blur)': '_focusChanged(false)',
                    '(focus)': '_focusChanged(true)',
                },
                providers: [
                    { provide: NtFormFieldControl, useExisting: NtInputDirective }
                ]
            },] }
];
NtInputDirective.ctorParameters = () => [
    { type: NgControl, decorators: [{ type: Optional }, { type: Self }] },
    { type: Platform },
    { type: ElementRef }
];
NtInputDirective.propDecorators = {
    placeholder: [{ type: Input }],
    disabled: [{ type: Input }],
    required: [{ type: Input }],
    type: [{ type: Input }],
    value: [{ type: Input }],
    readonly: [{ type: Input }]
};

class NtInputModule {
}
NtInputModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                exports: [NtInputDirective, NtInputGroupComponent, NtInputAddonComponent],
                declarations: [NtInputDirective, NtInputGroupComponent, NtInputAddonComponent],
                providers: [Platform]
            },] }
];

export { NtInputAddonComponent, NtInputDirective, NtInputGroupComponent, NtInputModule };
//# sourceMappingURL=input.js.map
