import { CommonModule } from '@angular/common';
import { EventEmitter, TemplateRef, Component, ViewEncapsulation, ElementRef, Input, Output, ViewChild, NgModule } from '@angular/core';
import { NtButtonModule } from '@ng-tangram/components/button';
import { NtOverlayPosition, NtOverlayComponent, NtOverlayModule } from '@ng-tangram/components/overlay';
import { CdkOverlayOrigin } from '@angular/cdk/overlay';

class NtPopConfirmComponent {
    constructor(_elementRef) {
        this._elementRef = _elementRef;
        this._title = '';
        this.position = NtOverlayPosition.Top;
        this.confirmText = '确认';
        this.cancelText = '取消';
        this.confirm = new EventEmitter();
        this.cancel = new EventEmitter();
        this.afterOpen = new EventEmitter();
        this.afterClosed = new EventEmitter();
        this.beforeOpen = new EventEmitter();
        this.beforeClosed = new EventEmitter();
        this.positionChange = new EventEmitter();
        this.origin = new CdkOverlayOrigin(this._elementRef);
    }
    set title(value) { this._title = value; }
    get title() { return this._title; }
    set template(value) { this._template = value; }
    get template() { return this._template; }
    set popconfirm(value) {
        if (value instanceof TemplateRef) {
            this._template = value;
        }
        else {
            this._title = value;
            this._template = null;
        }
    }
    ngOnChanges(changes) {
        const change = changes.title || changes.template || changes.popover;
        if (change && !change.firstChange) {
            Promise.resolve().then(() => {
                this.overlay.forceUpdatePosition();
            });
        }
    }
    _closeOverlay(event, isConfirm) {
        (isConfirm ? this.confirm : this.cancel).emit();
        this.overlay.close();
        event.stopPropagation();
    }
    _afterOpen(event) {
        this.afterOpen.next(event);
    }
    _afterClosed(event) {
        this.afterClosed.next(event);
    }
    _beforeOpen(event) {
        this.beforeOpen.next(event);
    }
    _beforeClosed(event) {
        this.beforeClosed.next(event);
    }
    _positionChange(change) {
        this.positionChange.next(change);
    }
}
NtPopConfirmComponent.decorators = [
    { type: Component, args: [{
                selector: '[nt-popconfirm]',
                template: "<ng-content></ng-content>\n<nt-overlay\n  overlayClass=\"nt-popconfirm-overlay\"\n  arrow\n  [origin]=\"origin\"\n  [position]=\"position\"\n  (afterOpen)=\"_afterOpen($event)\"\n  (afterClosed)=\"_afterClosed($event)\"\n  (beforeOpen)=\"_beforeOpen($event)\"\n  (beforeClosed)=\"_beforeClosed($event)\"\n  (positionChange)=\"_positionChange($event)\">\n  <div class=\"nt-popconfirm\">\n    <div class=\"nt-popconfirm-title\" *ngIf=\"template\"><ng-container *ngTemplateOutlet=\"template\"></ng-container></div>\n    <div class=\"nt-popconfirm-title\" *ngIf=\"!template\" [innerHtml]=\"title\"></div>\n    <div class=\"nt-popconfirm-action\">\n      <button nt-button color=\"secondary\" size=\"tiny\" (click)=\"_closeOverlay($event, false)\">{{cancelText}}</button>\n      <button nt-button size=\"tiny\" (click)=\"_closeOverlay($event, true)\">{{confirmText}}</button>\n    </div>\n  </div>\n</nt-overlay>\n",
                encapsulation: ViewEncapsulation.None,
                host: {
                    '(click)': 'overlay.toggle()'
                }
            },] }
];
NtPopConfirmComponent.ctorParameters = () => [
    { type: ElementRef }
];
NtPopConfirmComponent.propDecorators = {
    title: [{ type: Input }],
    template: [{ type: Input }],
    popconfirm: [{ type: Input, args: ['nt-popconfirm',] }],
    position: [{ type: Input }],
    confirmText: [{ type: Input }],
    cancelText: [{ type: Input }],
    confirm: [{ type: Output }],
    cancel: [{ type: Output }],
    afterOpen: [{ type: Output }],
    afterClosed: [{ type: Output }],
    beforeOpen: [{ type: Output }],
    beforeClosed: [{ type: Output }],
    positionChange: [{ type: Output }],
    overlay: [{ type: ViewChild, args: [NtOverlayComponent, { static: true },] }]
};

class NtPopConfirmModule {
}
NtPopConfirmModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, NtOverlayModule, NtButtonModule],
                exports: [NtPopConfirmComponent],
                declarations: [NtPopConfirmComponent]
            },] }
];

export { NtPopConfirmComponent, NtPopConfirmModule };
//# sourceMappingURL=popconfirm.js.map
