import { ContentObserver, ObserversModule } from '@angular/cdk/observers';
import { CommonModule } from '@angular/common';
import { InjectionToken, Component, ViewEncapsulation, ElementRef, Inject, EventEmitter, TemplateRef, Input, Output, ViewChild, NgModule } from '@angular/core';
import { NtOverlayPosition, NtOverlayComponent, NtOverlayModule } from '@ng-tangram/components/overlay';
import { NtDropdownModule } from '@ng-tangram/components/dropdown';
import { Subject } from 'rxjs';
import { CdkOverlayOrigin } from '@angular/cdk/overlay';

const NT_POPOVER_PARENT_COMPONENT = new InjectionToken('nt-popover-parent-component');
class NtPopoverPaneComponent {
    constructor(_contentObserver, _elementRef, _parent) {
        this._contentObserver = _contentObserver;
        this._elementRef = _elementRef;
        this._parent = _parent;
        this._contentChange = new Subject();
    }
    get contentChanged() {
        return this._contentChange.asObservable();
    }
    get textContent() {
        return (this._elementRef.nativeElement.textContent || '').trim();
    }
    ngAfterContentInit() {
        this._contentSubscription = this._contentObserver
            .observe(this._elementRef)
            .subscribe(() => this._checkContentChange());
    }
    ngOnDestroy() {
        if (this._contentSubscription) {
            this._contentSubscription.unsubscribe();
        }
    }
    _checkContentChange() {
        if (this._parent.overlay.opened) {
            this._parent.overlay.forceUpdatePosition();
        }
    }
}
NtPopoverPaneComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-popover-pane, [nt-popover-pane]',
                template: `
    <ng-content></ng-content>
  `,
                encapsulation: ViewEncapsulation.None
            },] }
];
NtPopoverPaneComponent.ctorParameters = () => [
    { type: ContentObserver },
    { type: ElementRef },
    { type: undefined, decorators: [{ type: Inject, args: [NT_POPOVER_PARENT_COMPONENT,] }] }
];

class NtPopoverComponent {
    constructor(_elementRef) {
        this._elementRef = _elementRef;
        this._title = '';
        this.position = NtOverlayPosition.Top;
        this.confirm = new EventEmitter();
        this.cancel = new EventEmitter();
        this.afterOpen = new EventEmitter();
        this.afterClosed = new EventEmitter();
        this.beforeOpen = new EventEmitter();
        this.beforeClosed = new EventEmitter();
        this.positionChange = new EventEmitter();
        this.origin = new CdkOverlayOrigin(this._elementRef);
    }
    set title(value) { this._title = value; }
    get title() { return this._title; }
    set template(value) { this._template = value; }
    get template() { return this._template; }
    set popover(value) {
        if (value instanceof TemplateRef) {
            this._template = value;
        }
        else {
            this._title = value;
            this._template = null;
        }
    }
    _afterOpen(event) {
        this.afterOpen.next(event);
    }
    _afterClosed(event) {
        this.afterClosed.next(event);
    }
    _beforeOpen(event) {
        this.beforeOpen.next(event);
    }
    _beforeClosed(event) {
        this.beforeClosed.next(event);
    }
    _positionChange(change) {
        this.positionChange.next(change);
    }
}
NtPopoverComponent.decorators = [
    { type: Component, args: [{
                selector: '[nt-popover]',
                template: "<ng-content></ng-content>\n<nt-overlay\n  overlayClass=\"nt-popover-overlay\"\n  arrow\n  [origin]=\"origin\"\n  [position]=\"position\"\n  (afterOpen)=\"_afterOpen($event)\"\n  (afterClosed)=\"_afterClosed($event)\"\n  (beforeOpen)=\"_beforeOpen($event)\"\n  (beforeClosed)=\"_beforeClosed($event)\"\n  (positionChange)=\"_positionChange($event)\">\n  <div class=\"nt-popover\">\n    <div class=\"nt-popover-title\" *ngIf=\"template\">\n      <ng-container *ngTemplateOutlet=\"template\"></ng-container>\n    </div>\n    <div class=\"nt-popover-title\" *ngIf=\"!template\" [innerHtml]=\"title\"></div>\n    <div class=\"nt-popover-pane\">\n      <ng-content select=\"nt-popover-pane\"></ng-content>\n    </div>\n  </div>\n</nt-overlay>\n",
                encapsulation: ViewEncapsulation.None,
                host: {
                    '(click)': 'overlay.toggle()'
                },
                providers: [
                    { provide: NT_POPOVER_PARENT_COMPONENT, useExisting: NtPopoverComponent }
                ]
            },] }
];
NtPopoverComponent.ctorParameters = () => [
    { type: ElementRef }
];
NtPopoverComponent.propDecorators = {
    title: [{ type: Input }],
    template: [{ type: Input }],
    popover: [{ type: Input, args: ['nt-popover',] }],
    position: [{ type: Input }],
    confirm: [{ type: Output }],
    cancel: [{ type: Output }],
    afterOpen: [{ type: Output }],
    afterClosed: [{ type: Output }],
    beforeOpen: [{ type: Output }],
    beforeClosed: [{ type: Output }],
    positionChange: [{ type: Output }],
    overlay: [{ type: ViewChild, args: [NtOverlayComponent, { static: true },] }]
};

class NtPopoverModule {
}
NtPopoverModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, ObserversModule, NtOverlayModule, NtDropdownModule],
                exports: [NtPopoverComponent, NtPopoverPaneComponent],
                declarations: [NtPopoverComponent, NtPopoverPaneComponent]
            },] }
];

export { NT_POPOVER_PARENT_COMPONENT, NtPopoverComponent, NtPopoverModule, NtPopoverPaneComponent };
//# sourceMappingURL=popover.js.map
