import { Component, Input, NgModule } from '@angular/core';
import { coerceNumberProperty } from '@angular/cdk/coercion';

class NtProgressCircleComponent {
    constructor() {
        this._max = 100;
        this._value = 0;
        this._radius = 60;
        this.size = 'medium';
        this.color = '';
        this.class = '';
    }
    get dasharray() {
        return this.radius * 2 * Math.PI;
    }
    get width() { return this.radius * 2 + 20; }
    get height() { return this.radius * 2 + 20; }
    set radius(value) { this._radius = coerceNumberProperty(value, 100); }
    get radius() { return this._radius; }
    set max(value) { this._max = coerceNumberProperty(value, 100); }
    get max() { return this._max; }
    set value(value) { this._value = coerceNumberProperty(value); }
    get value() { return this._value; }
    get percent() {
        const percent = this.value / this.max * 100;
        return percent > 100 ? 100 : percent;
    }
}
NtProgressCircleComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-progress-circle',
                template: "<div class=\"progress-container\">\n  <svg attr.viewPort=\"0 0 {{width / 2}} {{height / 2}}\" version=\"1.1\" xmlns=\"http://www.w3.org/2000/svg\"\n    [attr.width]=\"width\" [attr.height]=\"height\">\n    <circle\n      [attr.r]=\"radius\"\n      [attr.cx]=\"width / 2\"\n      [attr.cy]=\"height / 2\"\n      fill=\"transparent\"\n      [attr.stroke-dasharray]=\"dasharray\"\n      [attr.stroke-dashoffset]=\"0\">\n    </circle>\n\n    <circle\n      class=\"progress-meter-circle\"\n      [attr.r]=\"radius\"\n      [attr.cx]=\"width / 2\"\n      [attr.cy]=\"height / 2\"\n      fill=\"transparent\"\n      [attr.stroke-dasharray]=\"dasharray\"\n      [attr.stroke-dashoffset]=\"dasharray - (dasharray / 100 * percent)\">\n    </circle>\n  </svg>\n  <div class=\"progress-inner\">\n    <ng-content></ng-content>\n  </div>\n</div>\n<!-- <div class=\"nt-progress-circle-inner\"><ng-content></ng-content></div> -->\n",
                host: {
                    '[class]': '["nt-progress-circle", color, size, class].join(" ")'
                }
            },] }
];
NtProgressCircleComponent.propDecorators = {
    radius: [{ type: Input }],
    max: [{ type: Input }],
    value: [{ type: Input }],
    size: [{ type: Input }],
    color: [{ type: Input }],
    class: [{ type: Input }]
};

class NtProgressComponent {
    constructor() {
        this._max = 100;
        this._value = 0;
        this.size = 'medium';
        this.color = '';
        this.class = '';
    }
    set max(value) { this._max = coerceNumberProperty(value, 100); }
    get max() { return this._max; }
    set value(value) { this._value = coerceNumberProperty(value); }
    get value() { return this._value; }
    get percent() {
        const percent = this.value / this.max * 100;
        return percent > 100 ? 100 : percent;
    }
}
NtProgressComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-progress',
                template: "<span class=\"progress-meter\" [style.width.%]=\"percent > 100 ? 100 : percent\">\n  <div class=\"progress-meter-text\"><ng-content></ng-content></div>\n</span>\n",
                host: {
                    '[class]': '["progress", color, size, class].join(" ")'
                }
            },] }
];
NtProgressComponent.propDecorators = {
    max: [{ type: Input }],
    value: [{ type: Input }],
    size: [{ type: Input }],
    color: [{ type: Input }],
    class: [{ type: Input }]
};

class NtProgressModule {
}
NtProgressModule.decorators = [
    { type: NgModule, args: [{
                exports: [NtProgressComponent, NtProgressCircleComponent],
                declarations: [NtProgressComponent, NtProgressCircleComponent]
            },] }
];

export { NtProgressCircleComponent, NtProgressComponent, NtProgressModule };
//# sourceMappingURL=progress.js.map
