import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { EventEmitter, Component, ViewEncapsulation, Attribute, Self, Optional, Input, Output, isDevMode, ChangeDetectionStrategy, NgZone, ChangeDetectorRef, ContentChildren, NgModule } from '@angular/core';
import { NgControl } from '@angular/forms';
import { Subject, defer, merge } from 'rxjs';
import { take, switchMap, startWith, takeUntil } from 'rxjs/operators';
import { NtFormFieldControl } from '@ng-tangram/components/forms';
import { CommonModule } from '@angular/common';

let uniqueId = 0;
class NtRadioChange {
    constructor(source, selected) {
        this.source = source;
        this.selected = selected;
    }
}
class NtRadioComponent {
    constructor(tabIndex, ngControl) {
        this.ngControl = ngControl;
        this.id = `nt-radio-${uniqueId++}`;
        this._disabled = false;
        this._readonly = false;
        this._checked = false;
        this._name = this.id;
        this.change = new EventEmitter();
        this._onChange = () => { };
        this._onTouched = () => { };
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
        this.tabIndex = parseInt(tabIndex) || 0;
    }
    set value(value) { this._value = value; }
    get value() { return this._value; }
    set disabled(value) { this._disabled = coerceBooleanProperty(value); }
    get disabled() { return this._disabled; }
    set readonly(value) { this._readonly = coerceBooleanProperty(value); }
    get readonly() { return this._readonly; }
    get checked() { return this._checked; }
    set checked(value) { this._checked = coerceBooleanProperty(value); }
    set name(value) { this._name = value; }
    get name() { return this._name; }
    writeValue(value) {
        this._checked = !!value;
    }
    registerOnChange(fn) {
        this._onChange = fn;
    }
    registerOnTouched(fn) {
        this._onTouched = fn;
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
    }
    _onInputClick(event) {
        event.stopPropagation();
    }
    _onInputChange(event) {
        event.stopPropagation();
        this._checked = true;
        this._emitChangeEvent();
    }
    _emitChangeEvent() {
        const event = new NtRadioChange(this, this.checked);
        this._onChange(this.checked);
        this._onTouched();
        this.change.emit(event);
    }
}
NtRadioComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-radio',
                encapsulation: ViewEncapsulation.None,
                template: "<label class=\"nt-radio-layout\">\n  <div class=\"nt-radio-input-container\">\n    <input class=\"cdk-visually-hidden\"\n      type=\"radio\"\n      [name]=\"name\"\n      [tabIndex]=\"tabIndex\"\n      [disabled]=\"disabled\"\n      [checked]=\"checked\"\n      (change)=\"_onInputChange($event)\"\n      (click)=\"_onInputClick($event)\">\n    <div class=\"nt-radio-input\"></div>\n  </div>\n  <span class=\"nt-radio-label\">\n    <ng-content></ng-content>\n  </span>\n</label>\n",
                host: {
                    'class': 'nt-radio',
                    '[class.nt-radio-disabled]': 'disabled'
                }
            },] }
];
NtRadioComponent.ctorParameters = () => [
    { type: String, decorators: [{ type: Attribute, args: ['tabindex',] }] },
    { type: NgControl, decorators: [{ type: Self }, { type: Optional }] }
];
NtRadioComponent.propDecorators = {
    value: [{ type: Input }],
    disabled: [{ type: Input }],
    readonly: [{ type: Input }],
    checked: [{ type: Input }],
    name: [{ type: Input }],
    change: [{ type: Output }]
};

function getNtRdioGroupNonFunctionValueError() {
    return Error('`compareWith` must be a function.');
}
let uniqueId$1 = 0;
class NtRadioGroupComponent extends NtFormFieldControl {
    constructor(_ngZone, _changeDetectorRef, ngControl) {
        super();
        this._ngZone = _ngZone;
        this._changeDetectorRef = _changeDetectorRef;
        this.ngControl = ngControl;
        this.id = `nt-radio-group-${uniqueId$1++}`;
        this._destroy = new Subject();
        this._disabled = false;
        this._readonly = false;
        this._required = false;
        this._name = this.id;
        this._compareWith = (o1, o2) => o1 === o2;
        this.checkedChanges = defer(() => {
            if (this.radios) {
                return merge(...this.radios.map(item => item.change));
            }
            return this._ngZone.onStable
                .asObservable()
                .pipe(take(1), switchMap(() => this.checkedChanges));
        });
        this._onChange = () => { };
        this._onTouched = () => { };
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
    }
    get value() { return this._value; }
    set disabled(value) { this._disabled = coerceBooleanProperty(value); }
    get disabled() { return this._disabled; }
    get required() { return this._required; }
    set required(value) { this._required = coerceBooleanProperty(value); }
    set readonly(value) { this._readonly = coerceBooleanProperty(value); }
    get readonly() { return this._readonly; }
    get name() { return this._name; }
    set name(value) {
        this._name = value;
        this._updateRadioButtonNames();
    }
    set compareWith(fn) {
        if (typeof fn !== 'function') {
            throw getNtRdioGroupNonFunctionValueError();
        }
        else {
            this._compareWith = fn;
        }
    }
    ngAfterViewInit() {
        this.radios.changes.pipe(startWith(null), takeUntil(this._destroy)).subscribe(() => {
            this._resetRadios();
            this._initializeChecked();
        });
    }
    ngOnDestroy() {
        this._destroy.next();
        this._destroy.complete();
    }
    writeValue(value) {
        if (this.radios) {
            this._setCheckedByValue(value);
        }
    }
    registerOnChange(fn) {
        this._onChange = fn;
    }
    registerOnTouched(fn) {
        this._onTouched = fn;
    }
    focus() {
        if (!this.disabled) {
            this._onTouched();
        }
    }
    _initializeChecked() {
        Promise.resolve().then(() => {
            this._updateRadioButtonNames();
            this._setCheckedByValue(this.ngControl ? this.ngControl.value : this.value);
        });
    }
    _resetRadios() {
        const changedOrDestroyed = merge(this.radios.changes, this._destroy);
        this.checkedChanges.pipe(takeUntil(changedOrDestroyed)).subscribe(change => {
            this._setValues(change.source);
        });
        merge(...this.radios.map(item => item.change))
            .pipe(takeUntil(changedOrDestroyed))
            .subscribe(() => {
            this._changeDetectorRef.markForCheck();
        });
    }
    _setCheckedByValue(value) {
        this._value = value;
        this.radios.forEach(item => item.checked = false);
        this._checkedValue(value);
        this._changeDetectorRef.markForCheck();
    }
    _checkedValue(value) {
        const correspondingItem = this.radios.find((item) => {
            try {
                return item.value != null && this._compareWith(item.value, value);
            }
            catch (error) {
                if (isDevMode()) {
                    console.warn(error);
                }
                return false;
            }
        });
        if (correspondingItem) {
            correspondingItem.checked = true;
        }
        return correspondingItem;
    }
    _setValues(radio) {
        this._value = radio.value;
        this._onChange(this._value);
        this._onTouched();
        this._changeDetectorRef.markForCheck();
    }
    _updateRadioButtonNames() {
        if (this.radios) {
            this.radios.forEach(radio => {
                radio.name = this.name;
            });
        }
    }
}
NtRadioGroupComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-radio-group',
                template: '<ng-content></ng-content>',
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [
                    { provide: NtFormFieldControl, useExisting: NtRadioGroupComponent }
                ],
                host: {
                    'class': 'nt-radio-group'
                }
            },] }
];
NtRadioGroupComponent.ctorParameters = () => [
    { type: NgZone },
    { type: ChangeDetectorRef },
    { type: NgControl, decorators: [{ type: Self }, { type: Optional }] }
];
NtRadioGroupComponent.propDecorators = {
    disabled: [{ type: Input }],
    required: [{ type: Input }],
    readonly: [{ type: Input }],
    name: [{ type: Input }],
    radios: [{ type: ContentChildren, args: [NtRadioComponent,] }],
    compareWith: [{ type: Input }],
    checkedChanges: [{ type: Output }]
};

class NtRadioModule {
}
NtRadioModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                exports: [NtRadioComponent, NtRadioGroupComponent],
                declarations: [NtRadioComponent, NtRadioGroupComponent],
            },] }
];

export { NtRadioChange, NtRadioComponent, NtRadioGroupComponent, NtRadioModule, getNtRdioGroupNonFunctionValueError };
//# sourceMappingURL=radio.js.map
