import { CdkCellDef, CdkHeaderCellDef, CdkFooterCellDef, CdkColumnDef, CdkHeaderCell, CdkFooterCell, CdkCell, CdkHeaderRowDef, CdkFooterRowDef, CdkRowDef, CdkHeaderRow, CDK_ROW_TEMPLATE, CdkFooterRow, CdkRow, CdkTable, CDK_TABLE_TEMPLATE, _CoalescedStyleScheduler, CDK_TABLE, CdkTableModule, DataSource } from '@angular/cdk/table';
import { DOCUMENT, CommonModule } from '@angular/common';
import { Directive, EventEmitter, Input, ElementRef, Component, ChangeDetectionStrategy, ViewEncapsulation, NgZone, IterableDiffers, ChangeDetectorRef, Attribute, Optional, Inject, ContentChildren, Output, NgModule } from '@angular/core';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { Subject, defer, merge, BehaviorSubject, Subscription, combineLatest } from 'rxjs';
import { take, switchMap, startWith, takeUntil, filter, map } from 'rxjs/operators';
import { Directionality } from '@angular/cdk/bidi';
import { Platform } from '@angular/cdk/platform';

var NtColumnSort;
(function (NtColumnSort) {
    NtColumnSort["ASC"] = "asc";
    NtColumnSort["DESC"] = "desc";
    NtColumnSort["NONE"] = "";
})(NtColumnSort || (NtColumnSort = {}));
class NtColumnSortChange {
    constructor(isUserInput = false, column, sort) {
        this.isUserInput = isUserInput;
        this.column = column;
        this.sort = sort;
    }
}
class NtCellDefDirective extends CdkCellDef {
}
NtCellDefDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ntCellDef]',
                providers: [{ provide: CdkCellDef, useExisting: NtCellDefDirective }]
            },] }
];
class NtHeaderCellDefDirective extends CdkHeaderCellDef {
}
NtHeaderCellDefDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ntHeaderCellDef]',
                providers: [{ provide: CdkHeaderCellDef, useExisting: NtHeaderCellDefDirective }]
            },] }
];
class NtFooterCellDefDirective extends CdkFooterCellDef {
}
NtFooterCellDefDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ntFooterCellDef]',
                providers: [{ provide: CdkFooterCellDef, useExisting: NtFooterCellDefDirective }]
            },] }
];
class NtColumnDirective extends CdkColumnDef {
    constructor() {
        super(...arguments);
        this.sort = NtColumnSort.NONE;
        this._sortable = false;
        this._sortChange = new EventEmitter();
    }
    set column(value) { this.name = value; }
    set sortable(value) { this._sortable = coerceBooleanProperty(value); }
    get sortable() { return this._sortable; }
    sorting(isUserInput = false) {
        if (this.sortable) {
            switch (this.sort) {
                case NtColumnSort.ASC:
                    this.sort = NtColumnSort.DESC;
                    break;
                case NtColumnSort.DESC:
                    this.sort = NtColumnSort.NONE;
                    break;
                default:
                    this.sort = NtColumnSort.ASC;
                    break;
            }
            this._sortChange.emit(new NtColumnSortChange(isUserInput, this.name, this.sort));
        }
    }
}
NtColumnDirective.decorators = [
    { type: Directive, args: [{
                selector: 'nt-column, [nt-column]',
                providers: [{ provide: CdkColumnDef, useExisting: NtColumnDirective }],
            },] }
];
NtColumnDirective.propDecorators = {
    column: [{ type: Input, args: ['nt-column',] }],
    name: [{ type: Input }],
    sticky: [{ type: Input }],
    stickyEnd: [{ type: Input }],
    sortable: [{ type: Input }]
};
class NtHeaderCellDirective extends CdkHeaderCell {
    constructor(columnDef, elementRef) {
        super(columnDef, elementRef);
        this.columnDef = columnDef;
        elementRef.nativeElement.classList.add(`nt-column-${columnDef.cssClassFriendlyName}`);
    }
}
NtHeaderCellDirective.decorators = [
    { type: Directive, args: [{
                selector: 'nt-header-cell, th[nt-header-cell]',
                host: {
                    'class': 'nt-header-cell',
                    '[class.nt-column-sortable]': 'columnDef.sortable',
                    '[class.asc]': 'columnDef.sort === "asc"',
                    '[class.desc]': 'columnDef.sort === "desc"',
                    '(click)': 'columnDef.sorting(true)',
                    'role': 'columnheader',
                },
            },] }
];
NtHeaderCellDirective.ctorParameters = () => [
    { type: NtColumnDirective },
    { type: ElementRef }
];
class NtFooterCellDirective extends CdkFooterCell {
    constructor(columnDef, elementRef) {
        super(columnDef, elementRef);
        elementRef.nativeElement.classList.add(`nt-column-${columnDef.cssClassFriendlyName}`);
    }
}
NtFooterCellDirective.decorators = [
    { type: Directive, args: [{
                selector: 'nt-footer-cell, td[nt-footer-cell]',
                host: {
                    'class': 'nt-footer-cell',
                    'role': 'gridcell',
                },
            },] }
];
NtFooterCellDirective.ctorParameters = () => [
    { type: NtColumnDirective },
    { type: ElementRef }
];
class NtCellDirective extends CdkCell {
    constructor(columnDef, elementRef) {
        super(columnDef, elementRef);
        elementRef.nativeElement.classList.add(`nt-column-${columnDef.cssClassFriendlyName}`);
    }
}
NtCellDirective.decorators = [
    { type: Directive, args: [{
                selector: 'nt-cell, td[nt-cell]',
                host: {
                    'class': 'nt-cell',
                    'role': 'gridcell',
                },
            },] }
];
NtCellDirective.ctorParameters = () => [
    { type: NtColumnDirective },
    { type: ElementRef }
];

class NtTableResizable {
    constructor() { }
}
NtTableResizable.decorators = [
    { type: Directive, args: [{
                selector: 'table[ntTableResizable], nt-table[ntTableResizable]'
            },] }
];
NtTableResizable.ctorParameters = () => [];

class NtHeaderRowDefDirective extends CdkHeaderRowDef {
}
NtHeaderRowDefDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ntHeaderRowDef]',
                providers: [{ provide: CdkHeaderRowDef, useExisting: NtHeaderRowDefDirective }],
                inputs: ['columns: ntHeaderRowDef', 'sticky: ntHeaderRowDefSticky'],
            },] }
];
class NtFooterRowDefDirective extends CdkFooterRowDef {
}
NtFooterRowDefDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ntFooterRowDef]',
                providers: [{ provide: CdkFooterRowDef, useExisting: NtFooterRowDefDirective }],
                inputs: ['columns: ntFooterRowDef', 'sticky: ntFooterRowDefSticky'],
            },] }
];
class NtRowDefDirective extends CdkRowDef {
}
NtRowDefDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ntRowDef]',
                providers: [{ provide: CdkRowDef, useExisting: NtRowDefDirective }],
                inputs: ['columns: ntRowDefColumns', 'when: ntRowDefWhen'],
            },] }
];
class NtHeaderRowComponent extends CdkHeaderRow {
}
NtHeaderRowComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-header-row, tr[nt-header-row]',
                template: CDK_ROW_TEMPLATE,
                host: {
                    'class': 'nt-header-row',
                    'role': 'row',
                },
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                exportAs: 'ntHeaderRow',
                providers: [{ provide: CdkHeaderRow, useExisting: NtHeaderRowComponent }]
            },] }
];
class NtFooterRowComponent extends CdkFooterRow {
}
NtFooterRowComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-footer-row, tr[nt-footer-row]',
                template: CDK_ROW_TEMPLATE,
                host: {
                    'class': 'nt-footer-row',
                    'role': 'row',
                },
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                exportAs: 'ntFooterRow',
                providers: [{ provide: CdkFooterRow, useExisting: NtFooterRowComponent }]
            },] }
];
class NtRowComponent extends CdkRow {
}
NtRowComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-row, tr[nt-row]',
                template: CDK_ROW_TEMPLATE,
                host: {
                    'class': 'nt-row',
                    'role': 'row',
                },
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                exportAs: 'ntRow',
                providers: [{ provide: CdkRow, useExisting: NtRowComponent }]
            },] }
];

class NtTableComponent extends CdkTable {
    constructor(_ngZone, _differs, _changeDetectorRef, _coalescedStyleScheduler, _elementRef, role, _dir, _document, _platform) {
        super(_differs, _changeDetectorRef, _coalescedStyleScheduler, _elementRef, role, _dir, _document, _platform);
        this._ngZone = _ngZone;
        this._differs = _differs;
        this._changeDetectorRef = _changeDetectorRef;
        this._coalescedStyleScheduler = _coalescedStyleScheduler;
        this._elementRef = _elementRef;
        this._dir = _dir;
        this._multiSortable = false;
        this._destroy = new Subject();
        this.columSortChanges = defer(() => {
            if (this._contentColumns) {
                return merge(...this._contentColumns.map(column => column._sortChange));
            }
            return this._ngZone.onStable
                .asObservable()
                .pipe(take(1), switchMap(() => this.columSortChanges));
        });
        this.sortChange = new EventEmitter();
    }
    get sort() { return this._sort; }
    set sort(value) {
        this._sort = value;
    }
    ngAfterContentInit() {
        this._contentColumns.changes.pipe(startWith(null), takeUntil(this._destroy)).subscribe(() => {
            this._resetOptions();
            this._changeDetectorRef.markForCheck();
        });
        this.columSortChanges.pipe(takeUntil(this._destroy)).subscribe(change => {
            this._setSortValue(change.column, change.sort);
        });
        if (this.sort) {
            const [column, sort] = this.sort.split(':');
            if (column) {
                this._checkSortInputAndSetValue(column, sort);
            }
        }
    }
    ngOnChanges(changes) {
        const change = changes.sortValue;
        if (change && !change.firstChange) {
            const [column, sort] = change.currentValue.split(':');
            if (column) {
                this._checkSortInputAndSetValue(column, sort);
            }
        }
    }
    _setSortValue(column, sort) {
        if (sort && ['asc', 'desc'].includes(sort)) {
            this._sort = `${column}:${sort}`;
        }
        else {
            this._sort = '';
        }
    }
    _checkSortInputAndSetValue(column, sort) {
        const findColumn = this._contentColumns.find(col => col.name === column);
        if (findColumn) {
            findColumn.sort = sort;
        }
    }
    _clearSort(filter) {
        this._contentColumns
            .filter(column => column.name !== (filter ? filter.column : ''))
            .forEach(column => column.sort = NtColumnSort.NONE);
        this._changeDetectorRef.markForCheck();
    }
    _resetOptions() {
        const changedOrDestroyed = merge(this._contentColumns.changes, this._destroy);
        this.columSortChanges
            .pipe(takeUntil(changedOrDestroyed), filter(event => event.isUserInput))
            .subscribe(event => {
            if (!this._multiSortable) {
                this._clearSort(event);
            }
            this.sortChange.emit(event);
        });
        merge(...this._contentColumns.map(column => column._sortChange))
            .pipe(takeUntil(changedOrDestroyed))
            .subscribe(() => this._changeDetectorRef.markForCheck());
    }
    ngOnDestroy() {
        this._destroy.next();
        this._destroy.complete();
        super.ngOnDestroy();
    }
}
NtTableComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-table, table[nt-table]',
                template: CDK_TABLE_TEMPLATE,
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [
                    _CoalescedStyleScheduler,
                    { provide: CdkTable, useExisting: NtTableComponent },
                    { provide: CDK_TABLE, useExisting: NtTableComponent },
                ],
                host: {
                    'class': 'nt-table'
                }
            },] }
];
NtTableComponent.ctorParameters = () => [
    { type: NgZone },
    { type: IterableDiffers },
    { type: ChangeDetectorRef },
    { type: _CoalescedStyleScheduler },
    { type: ElementRef },
    { type: String, decorators: [{ type: Attribute, args: ['role',] }] },
    { type: Directionality, decorators: [{ type: Optional }] },
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] }] },
    { type: Platform }
];
NtTableComponent.propDecorators = {
    sort: [{ type: Input }],
    _contentColumns: [{ type: ContentChildren, args: [NtColumnDirective,] }],
    sortChange: [{ type: Output }]
};

const EXPORTS_DECLARATIONS = [
    NtTableComponent,
    NtTableResizable,
    NtCellDefDirective,
    NtCellDirective,
    NtColumnDirective,
    NtFooterCellDefDirective,
    NtFooterCellDirective,
    NtHeaderCellDefDirective,
    NtHeaderCellDirective,
    NtFooterRowComponent,
    NtFooterRowDefDirective,
    NtHeaderRowComponent,
    NtHeaderRowDefDirective,
    NtRowComponent,
    NtRowDefDirective
];
class NtTableModule {
}
NtTableModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, CdkTableModule],
                exports: EXPORTS_DECLARATIONS,
                declarations: EXPORTS_DECLARATIONS
            },] }
];

class NtTableDataSource extends DataSource {
    constructor(initialData = []) {
        super();
        this._renderData = new BehaviorSubject([]);
        this._filter = new BehaviorSubject('');
        this._renderChangesSubscription = Subscription.EMPTY;
        this.filterPredicate = (data, filter) => {
            const accumulator = (currentTerm, key) => currentTerm + data[key];
            const dataStr = Object.keys(data).reduce(accumulator, '').toLowerCase();
            const transformedFilter = filter.trim().toLowerCase();
            return dataStr.indexOf(transformedFilter) !== -1;
        };
        this._data = new BehaviorSubject(initialData);
        this._updateChangeSubscription();
    }
    get data() { return this._data.value; }
    set data(data) { this._data.next(data); }
    get filter() { return this._filter.value; }
    set filter(filter) { this._filter.next(filter); }
    _updateChangeSubscription() {
        const dataStream = this._data;
        const filteredData = combineLatest(dataStream, this._filter)
            .pipe(map(([data]) => this._filterData(data)));
        this._renderChangesSubscription.unsubscribe();
        this._renderChangesSubscription = filteredData.subscribe(data => this._renderData.next(data));
    }
    _filterData(data) {
        this.filteredData =
            !this.filter ? data : data.filter(obj => this.filterPredicate(obj, this.filter));
        return this.filteredData;
    }
    connect() { return this._renderData; }
    disconnect() { }
}

export { NtCellDefDirective, NtCellDirective, NtColumnDirective, NtColumnSort, NtColumnSortChange, NtFooterCellDefDirective, NtFooterCellDirective, NtFooterRowComponent, NtFooterRowDefDirective, NtHeaderCellDefDirective, NtHeaderCellDirective, NtHeaderRowComponent, NtHeaderRowDefDirective, NtRowComponent, NtRowDefDirective, NtTableComponent, NtTableDataSource, NtTableModule, NtTableResizable };
//# sourceMappingURL=table.js.map
